@@ -5,19 +5,18 @@
 #' \code{create_PsmCurves} is a function for creating an object of class
 #' \code{\link{PsmCurves}}.
 #' @param object Fitted survival models.
-#' @param input_data An object of class "expanded_hesim_data" returned by 
+#' @param input_data An object of class `expanded_hesim_data` returned by 
 #' \code{\link{expand.hesim_data}}. Must be expanded by the data tables "strategies" and
 #' "patients". 
 #' @param n Number of random observations of the parameters to draw.
-#' @param point_estimate If \code{TRUE}, then the point estimates are returned and and no samples are drawn.
-#' @param bootstrap If TRUE, then \code{n} bootstrap replications are drawn by refitting the survival
-#'  models in \code{object} on resamples of the sample data; if FALSE, then the parameters for each survival
-#'  model are independently draw from multivariate normal distributions.  
-#' @param est_data A \code{data.table} or \code{data.frame} of estimation data 
+#' @param uncertainty Method determining how parameter uncertainty should be handled. See
+#'  documentation in [`create_params()`].
+#' @param est_data A `data.table` or `data.frame` of estimation data 
 #' used to fit survival models during bootstrap replications.
-#' @param ... Further arguments passed to or from other methods. Currently unused. 
-#' @return Returns an \code{\link{R6Class}} object of class \code{\link{PsmCurves}}.
-#' @seealso \code{\link{PsmCurves}}
+#' @param ... Further arguments passed to or from other methods. Passed to [`create_params.partsurvfit()`]
+#' when `object` is of class [`flexsurvreg_list`].
+#' @return Returns an `R6Class` object of class [`PsmCurves`].
+#' @seealso [`PsmCurves`]
 #' @export
 create_PsmCurves <- function(object, ...){
   UseMethod("create_PsmCurves", object)
@@ -25,14 +24,25 @@ create_PsmCurves <- function(object, ...){
  
 #' @export
 #' @rdname create_PsmCurves
-create_PsmCurves.flexsurvreg_list <- function(object, input_data, n = 1000, point_estimate = FALSE,
-                                              bootstrap = FALSE, est_data = NULL, ...){
-  if (bootstrap == TRUE & is.null(est_data)){
-    stop("If 'bootstrap' == TRUE, then 'est_data' cannot be NULL")
+create_PsmCurves.flexsurvreg_list <- function(object, input_data, n = 1000, 
+                                              uncertainty = c("normal", "bootstrap", "none"), 
+                                              est_data = NULL,
+                                               ...){
+  # For backwards compatibility until deprecated point_estimate argument is no longer supported
+  is_uncertainty_missing <- missing(uncertainty)
+  uncertainty <- deprecate_point_estimate(list(...)$point_estimate, uncertainty,
+                                          is_uncertainty_missing)
+  uncertainty <- deprecate_bootstrap(list(...)$bootstrap, uncertainty,
+                                     is_uncertainty_missing)
+  
+  # Code to always keep
+  uncertainty <- match.arg(uncertainty)
+  if (uncertainty == "bootstrap" & is.null(est_data)){
+    stop("If uncertainty == 'bootstrap', then est_data cannot be NULL")
   }
   psfit <- partsurvfit(object, est_data)
   input_mats <- create_input_mats(psfit, input_data, id_vars = c("strategy_id", "patient_id"))
-  params <- create_params(psfit, n = n, point_estimate = point_estimate, bootstrap = bootstrap)
+  params <- create_params(psfit, n = n, uncertainty = uncertainty)
   return(PsmCurves$new(input_data = input_mats, params = params))
 }
 
@@ -47,7 +57,7 @@ create_PsmCurves.params_surv_list <- function(object, input_data, ...){
 #' Partitioned survival curves
 #'
 #' @description
-#' Summarize `n-1` survival curves for an `N` state partitioned survival model.
+#' Summarize N-1 survival curves for an N-state partitioned survival model.
 #' @format An [R6::R6Class] object.
 #' @examples 
 #' library("flexsurv")
@@ -73,14 +83,14 @@ create_PsmCurves.params_surv_list <- function(object, input_data, ...){
 #' # Form PsmCurves
 #' surv_input_data <- expand(hesim_dat, by = c("strategies", "patients"))
 #' psm_curves <- create_PsmCurves(fits, input_data = surv_input_data, n = 3,
-#'                                bootstrap = TRUE, est_data = surv_est_data)
+#'                                uncertainty = "bootstrap", est_data = surv_est_data)
 #'
 #' # Summarize survival curves
 #' head(psm_curves$quantile(p = c(.25, .5, .75)))
 #' head(psm_curves$survival(t = seq(0, 3, by = .1)))
 #' head(psm_curves$rmst(t = c(2, 5)))
 #' 
-#' @seealso [Psm], [create_PsmCurves()]
+#' @seealso [`Psm`], [`create_PsmCurves()`]
 #' @export
 PsmCurves <- R6::R6Class("PsmCurves",
   private = list(
@@ -108,10 +118,10 @@ PsmCurves <- R6::R6Class("PsmCurves",
   ),                            
                               
   public = list(
-    #' @field params An object of class [params_surv_list].
+    #' @field params An object of class [`params_surv_list`].
     params = NULL,
     
-    #' @field input_data An object of class [input_mats]. Each row in `X` must
+    #' @field input_data An object of class [`input_mats`]. Each row in `X` must
     #' be a unique treatment strategy and patient.
     input_data = NULL,
 
@@ -144,12 +154,14 @@ PsmCurves <- R6::R6Class("PsmCurves",
     },
     
     #' @description
-    #' Predict the cumulative hazard function for each survival curve as a function of time.
+    #' Predict survival probabilities for each survival curve as a function of time.
     #' @param t  A numeric vector of times.
-    #' @return A `data.table` with columns `sample`, `strategy_id`,
-    #' `patient_id`, `grp_id`, `curve`, `t`, and `survival`.    
+    #' @return An object of class [`survival`].   
     survival = function(t){
-      return(private$summary(x = t, type = "survival"))
+      res <- private$summary(x = t, type = "survival")
+      setattr(res, "class", 
+              c("survival", "data.table", "data.frame"))
+      return(res)
     },
 
     #' @description
@@ -196,32 +208,41 @@ PsmCurves <- R6::R6Class("PsmCurves",
 #' @format An [R6::R6Class] object.
 #' @examples
 #' library("flexsurv")
+#' library("ggplot2")
+#' theme_set(theme_bw())
 #'
 #' # Simulation data
-#' strategies <- data.frame(strategy_id = c(1, 2, 3))
+#' strategies <- data.frame(strategy_id = c(1, 2, 3), 
+#'                          strategy_name = paste0("Strategy ", 1:3))
 #' patients <- data.frame(patient_id = seq(1, 3),
 #'                        age = c(45, 50, 60),
 #'                        female = c(0, 0, 1))
 #' states <- data.frame(state_id =  seq(1, 3),
-#'                      state_name = paste0("state", seq(1, 3)))
+#'                      state_name = paste0("State ", seq(1, 3)))
 #' hesim_dat <- hesim_data(strategies = strategies,
 #'                         patients = patients,
 #'                         states = states)
-#' n_samples <- 3
+#' labs <- c(
+#'   get_labels(hesim_dat),
+#'   list(curve = c("Endpoint 1" = 1,
+#'                 "Endpoint 2" = 2,
+#'                  "Endpoint 3" = 3))
+#' )
+#' n_samples <- 2
 #'
 #' # Survival models
 #' surv_est_data <- psm4_exdata$survival
-#' fit1 <- flexsurv::flexsurvreg(Surv(endpoint1_time, endpoint1_status) ~ age,
+#' fit1 <- flexsurv::flexsurvreg(Surv(endpoint1_time, endpoint1_status) ~ factor(strategy_id),
 #'                               data = surv_est_data, dist = "exp")
-#' fit2 <- flexsurv::flexsurvreg(Surv(endpoint2_time, endpoint2_status) ~ age,
+#' fit2 <- flexsurv::flexsurvreg(Surv(endpoint2_time, endpoint2_status) ~ factor(strategy_id),
 #'                               data = surv_est_data, dist = "exp")
-#' fit3 <- flexsurv::flexsurvreg(Surv(endpoint3_time, endpoint3_status) ~ age,
+#' fit3 <- flexsurv::flexsurvreg(Surv(endpoint3_time, endpoint3_status) ~ factor(strategy_id),
 #'                               data = surv_est_data, dist = "exp")
 #' fits <- flexsurvreg_list(fit1, fit2, fit3)
 #'
 #' surv_input_data <- expand(hesim_dat, by = c("strategies", "patients"))
 #' psm_curves <- create_PsmCurves(fits, input_data = surv_input_data,
-#'                                bootstrap = TRUE, est_data = surv_est_data, 
+#'                                uncertainty = "bootstrap", est_data = surv_est_data, 
 #'                                n = n_samples)
 #'
 #' # Cost model(s)
@@ -236,34 +257,36 @@ PsmCurves <- R6::R6Class("PsmCurves",
 #' utility_tbl <- stateval_tbl(tbl = data.frame(state_id = states$state_id,
 #'                                              min = psm4_exdata$utility$lower,
 #'                                              max = psm4_exdata$utility$upper),
-#'                             dist = "unif",
-#'                             hesim_data = hesim_dat)
-#' psm_utility <- create_StateVals(utility_tbl, n = n_samples)
+#'                             dist = "unif")
+#' psm_utility <- create_StateVals(utility_tbl, n = n_samples,
+#'                                 hesim_data = hesim_dat)
 #'
 #' # Partitioned survival decision model
 #' psm <- Psm$new(survival_models = psm_curves,
 #'                utility_model = psm_utility,
 #'                cost_models = list(medical = psm_costs_medical))
-#' psm$sim_survival(t = seq(0, 5, .05))
+#' psm$sim_survival(t = seq(0, 5, 1/12))
+#' autoplot(psm$survival_, labels = labs, ci = FALSE, ci_style = "ribbon")
 #' psm$sim_stateprobs()
+#' autoplot(psm$stateprobs_, labels = labs)
 #' psm$sim_costs(dr = .03)
 #' head(psm$costs_)
 #' head(psm$sim_qalys(dr = .03)$qalys_)
 #'
-#' @seealso [PsmCurves], [create_PsmCurves()]
+#' @seealso [`PsmCurves`], [`create_PsmCurves()`]
 #' @export
 Psm <- R6::R6Class("Psm",
   public = list(
     #' @field survival_models The survival models used to predict survival curves. Must be
-    #' an object of class [PsmCurves].
+    #' an object of class [`PsmCurves`].
     survival_models = NULL,
     
     #' @field utility_model The model for health state utility. Must be an object of
-    #' class [StateVals].
+    #' class [`StateVals`].
     utility_model = NULL,
     
     #' @field cost_models The models used to predict costs by health state. 
-    #' Must be a list of objects of class [StateVals], where each element of the 
+    #' Must be a list of objects of class [`StateVals`], where each element of the 
     #' list represents a different cost category.    
     cost_models = NULL,
     
@@ -274,7 +297,7 @@ Psm <- R6::R6Class("Psm",
     #' by the argument `t` in `$sim_curves()`.
     t_ = NULL,
     
-    #' @field survival_ Survival curves simulated using `sim_curves()`.
+    #' @field survival_ An object of class [survival] simulated using `sim_survival()`.
     survival_ = NULL,
     
     #' @field stateprobs_ An object of class [stateprobs] simulated using `$sim_stateprobs()`.
@@ -301,10 +324,10 @@ Psm <- R6::R6Class("Psm",
     },
     
     #' @description
-    #' Simulate survival curves as a function of time using `PsmCurves$sim_survival()`.
+    #' Simulate survival curves as a function of time using `PsmCurves$survival()`.
     #' @param t A numeric vector of times. The first element must be `0`.
-    #' @return An instance of `self` with simulated output from `PsmCurves$sim_survival()`
-    #' stored in `stateprobs_`.
+    #' @return An instance of `self` with simulated output from `PsmCurves$survival()`
+    #' stored in `survival_`.
     sim_survival = function(t){
       if (t[1] !=0){
         stop("The first element of 't' must be 0.", call. = FALSE)
@@ -314,6 +337,8 @@ Psm <- R6::R6Class("Psm",
       }
       self$survival_models$check()
       self$survival_ <- self$survival_models$survival(t)
+      setattr(self$survival_, "class", 
+              c("survival", "data.table", "data.frame"))
       self$t_ <- t
       self$stateprobs_ <- NULL
       invisible(self)
@@ -348,6 +373,8 @@ Psm <- R6::R6Class("Psm",
       self$stateprobs_ <- stateprobs[]
       setattr(self$stateprobs_, "class", 
               c("stateprobs", "data.table", "data.frame"))
+      setattr(self$stateprobs_, "size", 
+              c(get_size(self$survival_models), n_states = self$n_states))
       invisible(self)
     },
 
@@ -380,10 +407,12 @@ Psm <- R6::R6Class("Psm",
     
     #' @description
     #' Summarize costs and QALYs so that cost-effectiveness analysis can be performed. 
-    #' See [summarize_ce()].       
-    summarize = function() {
+    #' See [summarize_ce()]. 
+    #' @param by_grp If `TRUE`, then costs and QALYs are computed by subgroup. If
+    #' `FALSE`, then costs and QALYs are aggregated across all patients (and subgroups).      
+    summarize = function(by_grp = FALSE) {
       check_summarize(self)
-      return(summarize_ce(self$costs_, self$qalys_))
+      return(summarize_ce(self$costs_, self$qalys_, by_grp))
     }
   )
 )
\ No newline at end of file
