@@ -9,17 +9,12 @@
 #' values. See "Details" for specifics. 
 #' @param dist Probability distribution used to sample parameters for a 
 #' probabilistic sensitivity analysis (PSA). 
-#' @param hesim_data A [hesim_data] object. Required to specify 
-#' treatment strategies , patients,
-#'  and/or health states not included as columns
-#' in `tbl`, or, to match patients in `tbl` to groups.
-#'  Not required if `tbl` includes one row for each treatment strategy, patient, and
-#' health state combination. Patients are matched to groups by specifying both a 
-#' `patient_id` and a `grp_var` column in the `patients` table.
+#' @param hesim_data A [`hesim_data`] object. This argument is deprecated
+#' and should be passed to [`create_StateVals.stateval_tbl()`] instead.
 #' @param grp_var The name of the variable used to group patients.
 #' 
 #' @details 
-#' `tbl` is a `data.table` containing columns for treatment 
+#' `tbl` is a tabular object containing columns for treatment 
 #' strategies (`strategy_id`), patients (`patient_id`),
 #' health states (`state_id`), and/or the start of time intervals 
 #' (`time_start`). The table must contain at least one column
@@ -62,13 +57,9 @@
 #'  but does not need to contain them all.
 #'  
 #'  
-#' @return An object of class "stateval_tbl", which is a `data.table` of
-#' parameter values with attributes for `dist` and optionally 
-#' `strategy_id`, `patients`, `state_id`, and `grp_var`. `tbl` 
-#' is in the same format as described in "Details". `patients` is a 
-#' `data.table` with one column containing `patient_id` and 
-#' optionally a second column containing `grp_var`.
-#' @seealso [create_StateVals], [StateVals]
+#' @return An object of class `stateval_tbl`, which is a `data.table` of
+#' parameter values with attributes for `dist` and `grp_var`.
+#' @seealso [`create_StateVals()`], [`StateVals`]
 #' @examples 
 #' strategies <- data.frame(strategy_id = c(1, 2))
 #' patients <- data.frame(patient_id = seq(1, 3),
@@ -86,19 +77,17 @@
 #'                                        mean = c(.8, .7, .75, .55),
 #'                                        se = c(.18, .12, .10, .06)),
 #'                             dist = "beta",
-#'                             hesim_data = hesim_dat,
 #'                             grp_var = "grp")
 #' print(utility_tbl)
-#' utilmod <- create_StateVals(utility_tbl, n = 2)
+#' utilmod <- create_StateVals(utility_tbl, n = 2, hesim_data = hesim_dat)
 #'
 #' # Costs vary by treatment strategy
 #' cost_tbl <- stateval_tbl(data.frame(strategy_id = strategies$strategy_id,
 #'                                     mean = c(5000, 3000),
 #'                                     se = c(200, 100)),
-#'                          dist = "gamma",
-#'                          hesim_data = hesim_dat)
+#'                          dist = "gamma")
 #' print(cost_tbl)
-#' costmod <- create_StateVals(cost_tbl, n = 2)
+#' costmod <- create_StateVals(cost_tbl, n = 2, hesim_data = hesim_dat)
 #'
 #'
 #' @export
@@ -106,6 +95,9 @@ stateval_tbl <- function(tbl, dist = c("norm", "beta", "gamma",
                                        "lnorm", "unif", "fixed", "custom"),
                          hesim_data = NULL,
                          grp_var = NULL){
+  if (!missing("hesim_data")) {
+    warning("'hesim_data' argument deprecated; pass to create_StateVals() instead.")
+  }
   dist <- match.arg(dist)
   tbl <- data.table(tbl)
   tbl2 <- copy(tbl)
@@ -120,36 +112,6 @@ stateval_tbl <- function(tbl, dist = c("norm", "beta", "gamma",
   }
   
   # Check
-  ## Column names
-  check_column <- function(var){
-    if (is.null(tbl2[[var]])){
-      name <- switch(var,
-                     "state_id" = "states",
-                     "strategy_id" = "strategies",
-                     "patient_id" = "patients")
-      if (is.null(hesim_data[[name]])){
-        msg <- paste0("If '", var, "' is not a column in 'tbl', ",
-                      "then 'hesim_data' must be included as an argument ",
-                      "and '",  name, "' must be an element of 'hesim_data'.")
-        stop(msg, call. = FALSE)
-      }
-    }
-  }
-  check_column("state_id")
-  check_column("strategy_id")
-  check_column("patient_id")
-  
-  ### Additional logic for patients
-  if (is.null(tbl2[["patient_id"]])){
-    if (is.null(hesim_data[["patients"]][["patient_id"]]) & 
-        (is.null(grp_var) || is.null(tbl2[[grp_var]]))){
-      stop(paste0("If 'patient_id' is not included as a column in `tbl`, ",
-                  "then both 'patient_id' and 'grp_var' cannot be missing from the ",
-                  " 'patients' element of hesim_data"),
-           call. = FALSE)
-    }
-  }
-  
   ## Need sample
   if (dist != "custom") {
     if ("sample" %in% colnames(tbl2)){
@@ -245,9 +207,9 @@ stateval_tbl <- function(tbl, dist = c("norm", "beta", "gamma",
   expected_n_times <- size("time_start")
   expected_n <- expected_n_samples * expected_n_strategies * expected_n_states * expected_n_grps * expected_n_times
   if (nrow(tbl2) != expected_n) {
-    stop(paste0("The number of rows in 'tbl' should equal ", expected_n, 
+    stop(paste0("The number of rows in 'object' should equal ", expected_n, 
                 " which is the number of unique values of ",
-                id_vars_msg, " in 'tbl'"))
+                id_vars_msg, " in 'object'."))
   }
   
   # Nice sorting
@@ -259,10 +221,10 @@ stateval_tbl <- function(tbl, dist = c("norm", "beta", "gamma",
   # Return
   setattr(tbl2, "class", c("stateval_tbl", "data.table", "data.frame"))
   setattr(tbl2, "dist", dist)
-  setattr(tbl2, "strategy_id", hesim_data$strategies$strategy_id)
-  setattr(tbl2, "patients", data.table(hesim_data$patients))
-  setattr(tbl2, "state_id", hesim_data$states$state_id)
   setattr(tbl2, "grp_var", grp_var)
+  if (!is.null(hesim_data)) {
+    attr(tbl2, "hesim_data") <- hesim_data
+  }
   return(tbl2)
 }
 
@@ -270,17 +232,26 @@ stateval_tbl <- function(tbl, dist = c("norm", "beta", "gamma",
 #' Create a `StateVals` object
 #' 
 #' `create_StateVals()` is a generic function for creating an object of class
-#'  [StateVals] from a fitted statistical model or a [stateval_tbl]
+#'  [`StateVals`] from a fitted statistical model or a [`stateval_tbl`]
 #'  object. 
 #' @param object A model object of the appropriate class.
-#' @param input_data An object of class [expanded_hesim_data][expand.hesim_data()].
+#' @param input_data An object of class [`expanded_hesim_data`][expand.hesim_data()].
 #' Must be expanded by treatment strategies, patients, and health states.
+#' @param hesim_data A [`hesim_data`] object. Only required when `object` is of class
+#' [`stateval_tbl`]. See "details". 
 #' @param n Number of random observations of the parameters to draw when parameters 
 #' are fit using a statistical model.
-#' @param point_estimate If `TRUE`, then the point estimates are returned and and no samples are drawn.
-#' @param ... Further arguments (`time_reset` and `method`) passed to [StateVals$new()][StateVals].
-#' @return A [StateVals] object.
-#' @seealso [StateVals], [stateval_tbl]
+#' @param uncertainty Method determining how parameter uncertainty should be handled. See
+#'  documentation in [`create_params()`].
+#' @param ... Further arguments (`time_reset` and `method`) passed to [`StateVals$new()`][StateVals].
+#' @details If `object` is a `stateval_tbl`, then a [`hesim_data`] object is used
+#'  to specify treatment strategies, patients, and/or health states not included as 
+#'  columns in the table, or, to match patients in the table to groups. Not required if 
+#'  the table includes one row for each treatment strategy, patient, and health state
+#'   combination. Patients are matched to groups by specifying both a `patient_id` 
+#'   and a `grp_var` column in the `patients` table.
+#' @return A [`StateVals`] object.
+#' @seealso [`StateVals`], [`stateval_tbl()`]
 #' @export
 create_StateVals <- function(object, ...){
   UseMethod("create_StateVals", object)
@@ -289,15 +260,62 @@ create_StateVals <- function(object, ...){
 #' @rdname create_StateVals
 #' @export  
 create_StateVals.lm <- function(object, input_data = NULL, n = 1000,
-                                point_estimate = FALSE, ...){
-  params <- create_params(object, n, point_estimate) 
+                                uncertainty = c("normal", "none"), ...){
+  uncertainty <- deprecate_point_estimate(list(...)$point_estimate, uncertainty,
+                                          missing(uncertainty))
+  params <- create_params(object, n, uncertainty) 
   input_mats <- create_input_mats(object, input_data)
   return(StateVals$new(params = params, input_data = input_mats, ...))
 }
 
 #' @rdname create_StateVals 
 #' @export
-create_StateVals.stateval_tbl <- function(object, n = 1000, ...){
+create_StateVals.stateval_tbl <- function(object, hesim_data = NULL, n = 1000, ...){
+  
+  grp_var <- attr(object, "grp_var")
+  
+  # For backwards compatibility, use hesim_data attribute of object 
+  # if needed and available
+  if (!is.null(attr(object, "hesim_data")) && is.null(hesim_data)) {
+    hesim_data <- attr(object, "hesim_data")
+  } 
+  
+  # Check whether hesim_data is required
+  need_hesim_data <- function(var){
+    if (is.null(object[[var]])){
+      name <- switch(var,
+                     "state_id" = "states",
+                     "strategy_id" = "strategies",
+                     "patient_id" = "patients")
+      if (is.null(hesim_data[[name]])){
+        msg <- paste0("If '", var, "' is not a column in 'object', ",
+                      "then 'hesim_data' must be included as an argument ",
+                      "and '",  name, "' must be an element of 'hesim_data'.")
+        stop(msg, call. = FALSE)
+      }
+    }
+  }
+  need_hesim_data("state_id")
+  need_hesim_data("strategy_id")
+  need_hesim_data("patient_id")
+  
+  ## Additional logic for patients
+  if (is.null(object[["patient_id"]])){
+    if (is.null(hesim_data[["patients"]][["patient_id"]]) & 
+        (is.null(grp_var) || is.null(object[[grp_var]]))){
+      stop(paste0("If 'patient_id' is not included as a column in `object`, ",
+                  "then both 'patient_id' and 'grp_var' cannot be missing from the ",
+                  " 'patients' element of hesim_data"),
+           call. = FALSE)
+    }
+  }
+
+  # If not NULL, add hesim_data to attributes of object 
+  if (!is.null(hesim_data)) {
+    setattr(object, "strategy_id", hesim_data$strategies$strategy_id)
+    setattr(object, "patients", data.table(hesim_data$patients))
+    setattr(object, "state_id", hesim_data$states$state_id)
+  }
   
   # Random number generation
   tbl <- copy(object)
@@ -438,11 +456,11 @@ create_StateVals.eval_model <- function(object, cost = TRUE, name = NULL,
 StateVals <- R6::R6Class("StateVals",
   public = list(
     #' @field params Parameters for simulating state values. Currently supports
-    #'  objects of class [tparams_mean] or [params_lm].   
+    #'  objects of class [`tparams_mean`] or [`params_lm`].   
     params = NULL,
     
     #' @field input_data  An object of class [input_mats]. Only used for
-    #' [params_lm] objects.
+    #' [`params_lm`] objects.
     input_data = NULL,
     
     #' @field method The method used to simulate costs and 
@@ -535,29 +553,11 @@ sim_ev.stateprobs <- function(object, statevalmods, categories, dr = .03,
          call. = FALSE)
   }
   
-  # Discount rate
+  ## Discount rate
   check_dr(dr)
   
-  # The state value models
-  expected_samples <- max(object$sample)
-  method <- rep(NA, length(statevalmods))
-  for (i in 1:length(statevalmods)){
-    ## Number of samples
-    if (statevalmods[[i]]$params$n_samples != expected_samples){
-      msg <- paste0("Number of samples in each state value model must equal to ",
-                    " the number of samples in the 'stateprobs' object, which is ",
-                    expected_samples)
-      stop(msg, call. = FALSE)
-    }
- 
-    ## Number of states
-    if(length(unique(object$state_id)) != get_id_object(statevalmods[[i]])$n_states + 1){
-      msg <- paste0("The number of states in each 'StateVals' model ", 
-                    "must be one less (since state values cannot be applied to the ",
-                    "death state) than the number of states in 'stateprobs'.")
-      stop(msg, call. = FALSE)
-    }
-  }
+  ## The size of ID variables is correct
+  check_StateVals(statevalmods, object)
   
   # Simulate
   res <- data.table(C_sim_ev(object[state_id != max(state_id)],
@@ -589,10 +589,10 @@ sim_los <- function (object, utility_model, dr,
 #' Simulate costs and quality-adjusted life-years (QALYs) as a function of
 #' simulated state occupancy probabilities. 
 #' 
-#' @param object A [stateprobs] object.
-#' @param utility_model A single object of class [StateVals] used
+#' @param object A [`stateprobs`] object.
+#' @param utility_model A single object of class [`StateVals`] used
 #' to simulate utility.
-#' @param cost_models A list of objects of class [StateVals] used
+#' @param cost_models A list of objects of class [`StateVals`] used
 #' to simulate costs.
 #' @param dr Discount rate. 
 #' @param integrate_method Method used to integrate state values when computing 
@@ -602,8 +602,8 @@ sim_los <- function (object, utility_model, dr,
 #' @param lys If `TRUE`, then life-years are simulated in addition to 
 #' QALYs. 
 #' @keywords internal
-#' @return [sim_costs()] and [sim_qalys()] return objects of class
-#' [costs] and [qalys], respectively. 
+#' @return [`sim_costs()`] and [`sim_qalys()`] return objects of class
+#' [`costs`] and [`qalys`], respectively. 
 #' @details 
 #' See `vignette("expected-values")` for details.
 #'
@@ -651,76 +651,4 @@ sim_costs <- function(object, cost_models, dr, integrate_method){
   setattr(costs, "class", 
           c("costs", "data.table", "data.frame"))
   return(costs[, ])
-}
-
-#' Costs object
-#'
-#' An object of class `costs` returned from methods 
-#' `$sim_costs()` in model classes that store simulated costs. 
-#' 
-#' @section Components:
-#' A `costs` object inherits from `data.table` and contains
-#' the following columns:
-#' 
-#' \describe{
-#'   \item{sample}{A random sample from the PSA.}
-#'   \item{strategy_id}{The treatment strategy ID.}
-#'   \item{patient_id}{The patient ID.}
-#'   \item{state_id}{The health state ID.}
-#'   \item{dr}{The rate used to discount costs.}
-#'   \item{category}{The cost category (e.g., drug costs, medical costs, etc).}
-#'   \item{costs}{The simulated cost values.}
-#' }
-#'
-#' @name costs
-NULL
-
-#' Quality-adjusted life-years object
-#'
-#' An object of class `qalys` returned from methods 
-#' `$sim_qalys()` in model classes that store simulated 
-#' quality-adjusted life-years (QALYs).
-#' 
-#' @section Components:
-#' A `qalys` object inherits from `data.table` and contains
-#' the following columns:
-#' 
-#' \describe{
-#'   \item{sample}{A random sample from the PSA.}
-#'   \item{strategy_id}{The treatment strategy ID.}
-#'   \item{patient_id}{The patient ID.}
-#'   \item{state_id}{The health state ID.}
-#'   \item{dr}{The rate used to discount QALYs.}
-#'   \item{category}{A single category always equal to "qalys".}
-#'   \item{qalys}{The simulated values of QALYs.}
-#' }
-#' If the argument `lys = TRUE`, then the `data.table` also contains a column
-#' `lys` containing simulated life-years.
-#' @name qalys
-NULL
-
-# State probability object -----------------------------------------------------
-#' State probability object
-#'
-#' An object of class `stateprobs` returned from methods 
-#' `$sim_stateprobs()` in model classes. 
-#' 
-#' @section Components:
-#' A `stateprobs` object inherits from `data.table` and contains
-#' the following columns:
-#' 
-#' \describe{
-#'   \item{sample}{A random sample from the PSA.}
-#'   \item{strategy_id}{The treatment strategy ID.}
-#'   \item{patient_id}{The patient ID.}
-#'   \item{state_id}{The health state ID.}
-#'   \item{t}{The time at which a state probability is computed.}
-#'   \item{prob}{The probability of being in a given health state.}
-#' }
-#' 
-#' When simulating individual-level models, the `patient_id` column is
-#' not included as state probabilities are computed by averaging across patients.
-#'
-#'    
-#' @name stateprobs
-NULL
\ No newline at end of file
+}
\ No newline at end of file
