@@ -1,6 +1,7 @@
 context("dtstm.R unit tests")
 library("data.table")
 library("nnet")
+library("msm")
 rm(list = ls())
 
 # Helper functions -------------------------------------------------------------
@@ -95,7 +96,7 @@ tprob_array <- aperm(tprob_array, perm = c(6:3, 1, 2))
 params_tprob <- tparams_transprobs(tprob_array, times = time_start)
 params_tprob_t1 <- tparams_transprobs(tprob_array[,,,1,,, drop = FALSE])
 
-test_that("Extra arguments with tparams_transprobs.array" , {
+test_that("Extra arguments with tparams_transprobs.array()" , {
   expect_equal(tparams_transprobs(tprob_array, times = time_start, grp_id = 1)$grp_id,
                rep(1, prod(dim(tprob_array)[1:4])))
   expect_equal(tparams_transprobs(tprob_array, times = time_start, patient_wt = 1)$patient_wt,
@@ -111,13 +112,13 @@ test_that("Extra arguments with tparams_transprobs.array" , {
   
 })
 
-test_that("tparams_transprobs returns array of matrices" , {
+test_that("tparams_transprobs() returns array of matrices" , {
   expect_true(inherits(params_tprob$value, "array"))
   expect_equal(length(dim(params_tprob$value)), 3)
   expect_equal(dim(params_tprob$value)[1], dim(params_tprob$value)[2])
 })
 
-test_that("tparams_transprobs with only 1 time interval", {
+test_that("tparams_transprobs() with only 1 time interval", {
   expect_true(inherits(params_tprob_t1, "tparams_transprobs"))
   expect_equal(params_tprob_t1$n_times, 1)
   expect_equal(nrow(params_tprob_t1$time_intervals), 1)
@@ -127,20 +128,35 @@ test_that("tparams_transprobs with only 1 time interval", {
 # as.data.table.tparams_transprobs() -------------------------------------------
 tprob_dt <- as.data.table(params_tprob)
   
-test_that("as.data.table.tparams_transprobs returns a data.table" , {
+test_that("as.data.table.tparams_transprobs() returns a data.table" , {
   expect_true(inherits(as.data.table(params_tprob), "data.table"))
 })
 
 # tparams_transprobs.data.table() ----------------------------------------------
 params_tprob2 <- tparams_transprobs(tprob_dt)
 
-test_that(paste0("tparams_transprobs returns the same values with ",
+test_that(paste0("tparams_transprobs() returns the same values with ",
                  ".array and .data.table "), {
   expect_equal(params_tprob, params_tprob2)
   expect_equal(params_tprob_t1, 
                tparams_transprobs(tprob_dt[time_id == 1]))                
 })
 
+test_that("tparams_transprobs.data.table() checks ID attributes", {
+  expect_error(
+    tparams_transprobs(tprob_dt[1:5]),
+    paste0("The length of the ID variables is not consistent with the ",
+            "number of unique values of each ID variable.")
+  )
+})
+
+test_that("tparams_transprobs.data.table() throws error if there are no 'prob_' columns", {
+  expect_error(
+    tparams_transprobs(tprob_dt[, .(sample, strategy_id)]),
+    "No columns with names starting with 'prob_'."
+  )
+})
+
 # tparams_transprobs.tpmatrix() ------------------------------------------------
 p <- c(.7, .6)
 tpmat <- tpmatrix(
@@ -149,21 +165,21 @@ tpmat <- tpmatrix(
 )
 input_dat <- expand(hesim_dat)
 
-test_that("tparams_transprobs returns error if 'tpmatrix_id' has wrong class", {
+test_that("tparams_transprobs() returns error if 'tpmatrix_id' has wrong class", {
   expect_error(
     tparams_transprobs(tpmat, 2),
     "'tpmatrix_id' must be of class 'data.frame'." 
   )
 })
 
-test_that("tparams_transprobs returns error if 'tpmatrix_id' has wrong number of rows", {
+test_that("tparams_transprobs() returns error if 'tpmatrix_id' has wrong number of rows", {
   expect_error(
     tparams_transprobs(tpmat, data.frame(2)),
     "'object' and 'tpmatrix_id' must have the same number of rows." 
   )
 })
 
-test_that("tparams_transprobs returns the correct class", {
+test_that("tparams_transprobs() returns the correct class", {
   tpmat_id <- tpmatrix_id(input_dat, n_samples = 1)
   tp <- tpmatrix(
     C, c(.6, .7, .5, .4),
@@ -184,13 +200,13 @@ tpmat <- tpmatrix(
 tparray <- as_array3(tpmat)
 tpmat_id <- tpmatrix_id(input_dat, n_samples = 1)
 
-test_that("tparams_transprobs.array works as expected with 3D array", {
+test_that("tparams_transprobs.array() works as expected with 3D array", {
   tprob1 <- tparams_transprobs(tparray, tpmat_id)
   tprob2 <- tparams_transprobs(tpmat, tpmat_id)
   expect_equal(tprob1, tprob2)
 })
 
-test_that("tparams_transprobs.array returns an error with the incorrect number of slices", {
+test_that("tparams_transprobs.array() returns an error with the incorrect number of slices", {
   expect_error(tparams_transprobs(tparray[, , -1], tpmat_id),
                paste0("The third dimension of the array 'object' must equal ",
                       "the number or rows in 'tpmatrix_id'"))
@@ -199,11 +215,11 @@ test_that("tparams_transprobs.array returns an error with the incorrect number o
 # Initialize CohortDtstmTrans object -------------------------------------------
 transmod <- CohortDtstmTrans$new(params = params_tprob)
 
-test_that("CohortDtstmTrans$new automatically set 'start_stateprobs' ",{
+test_that("CohortDtstmTrans$new() automatically sets 'start_stateprobs' ",{
   expect_equal(transmod$start_stateprobs, c(1, 0, 0))   
 })
 
-test_that("CohortDtstmTrans$new 'start_stateprobs' normalizes to 1 ",{
+test_that("CohortDtstmTrans$new() 'start_stateprobs' normalizes to 1 ",{
   # Positive values
   v <- c(5, 5, 10, 10)
   tmp <- CohortDtstmTrans$new(params = params_tprob,
@@ -218,7 +234,7 @@ test_that("CohortDtstmTrans$new 'start_stateprobs' normalizes to 1 ",{
   expect_equal(tmp$start_stateprobs, c(1/2, 1/2))
 })
 
-test_that("CohortDtstmTrans$new 'start_stateprobs' exceptions ",{
+test_that("CohortDtstmTrans$new() 'start_stateprobs' exceptions ",{
   expect_error(CohortDtstmTrans$new(params = params_tprob, 
                                     start_stateprobs = c(Inf, 1)),
                "Elements of 'state_stateprobs' cannot be infinite.")
@@ -302,7 +318,7 @@ transmod <- create_CohortDtstmTrans(transfits,
                                     input_data = transmod_data,
                                     trans_mat = tmat,
                                     n = n_samples,
-                                    point_estimate = TRUE)
+                                    uncertainty = "none")
 
 
 test_that(paste0("create_CohortDtstmTrans$sim_stateprobs() is consistent with ",
@@ -377,8 +393,53 @@ test_that(paste0("create_CohortDtstmTrans does not support offset term ",
                "An offset is not supported")
 })
 
+# Create model from a msm object -----------------------------------------------
+set.seed(101)
+strategies <- data.table(strategy_id = c(1, 2, 3),
+                         strategy_name = factor(c("SOC", "New 1", "New 2")))
+patients <- data.table(patient_id = 1:2)
+hesim_dat <- hesim_data(strategies = strategies,
+                        patients = patients)
+transmod_data <- expand(hesim_dat)
+qinit <- rbind(
+  c(0, 0.28163, 0.01239),
+  c(0, 0, 0.10204),
+  c(0, 0, 0)
+)
+fit <- msm(state_id ~ time, subject = patient_id, 
+           data = onc3p[patient_id %in% sample(patient_id, 100)],
+           covariates = list("1-2" =~ strategy_name), 
+           qmatrix = qinit)
+
+test_that("create_CohortDtstmTrans.msm() returns correct transition probability matrices with no uncertainty", {
+ 
+  transmod <- create_CohortDtstmTrans(fit,
+                                      input_data = transmod_data,
+                                      cycle_length = 1/2,
+                                      fixedpars = 2,
+                                      uncertainty = "none")
+  expect_equal(transmod$params$n_samples, 1)
+  expect_equal(
+    expmat(qmatrix(fit, transmod_data, uncertainty = "none"), t = 1/2),
+    transmod$params$value
+  )
+})
+
+test_that(paste0("create_CohortDtstmTrans.msm() returns transition probability matrices",
+                 "with correct dimensions when there is uncertainty"), {
+
+  transmod <- create_CohortDtstmTrans(fit,
+                                      input_data = transmod_data,
+                                      cycle_length = 1/2,
+                                      fixedpars = 2,
+                                      n = 2)
+  expect_equal(transmod$params$n_samples, 2)
+  expect_equal(dim(transmod$params$value)[3], 2 * nrow(transmod_data))
+})
+
+
 # Create model from a model_def object -----------------------------------------
-test_that(paste0("Use define_model() to create CohortDtstmTrans object"), {
+test_that(paste0("define_model() works to create CohortDtstmTrans object"), {
   hesim_dat <- hesim_data(
     strategies = data.table(strategy_id = 1:2,
                             strategy_name = c("Monotherapy", "Combination therapy")),
